<?php
// $Id: mm_sitemap.inc 5139 2011-03-11 19:41:39Z vquinn $
/**
 * @file
 * Generate an XML sitemap based on Monster Menu permissions.
 */

$_mm_sitemap_exclude_list = &drupal_static('_mm_sitemap_exclude_list', array(
  '/-system',
  '/go',
  '/launch',
  '/myamherst',
));

/**
 * Use an iterator to generate the sitemap
 */
function mm_sitemap() {
  class dumpIter extends getTreeIterator {
    protected $fp, $file_path, $tree_path;

    public function __construct() {
      $this->file_path = file_default_scheme() . '://sitemap.xml';
      $this->fp = fopen($this->file_path . '.temp', 'w');
      if ($this->fp) {
        $this->tree_path = array();
        fwrite($this->fp, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<urlset xmlns=\"http://www.sitemaps.org/schemas/sitemap/0.9\">\n");
      }
      else {
        watchdog('mm', 'Could not create !path', array('!path' => $this->file_path . '.temp'), WATCHDOG_ERROR);
        exit();
      }
    }

    public function iterate($item) {
      global $base_url;
      $_mm_sitemap_exclude_list = &drupal_static('_mm_sitemap_exclude_list');

      // root level?
      if ($item->level <= 0)
        return 1;     // no error

      // not publicly readable, hidden, or recycled? not the normal menu block?
      if ($item->level > MM_SITEMAP_MAX_LEVEL || !$item->perms[MM_PERMS_READ] || $item->hidden || $item->perms[MM_PERMS_IS_RECYCLED] || $item->bid > 1)
        return -1;    // skip children

      $node_name = empty($item->alias) ? $item->mmtid : $item->alias;
      if ($item->level >= 2) {
        array_splice($this->tree_path, $item->level - 2);
        $this->tree_path[] = $node_name;
      }
      $path = '/' . join('/', $this->tree_path);

      // check the exclusions list
      if (array_search($path, $_mm_sitemap_exclude_list) !== FALSE)
        return -1;    // skip children

      // Figure out if the node or container is newer and use that date
      // Comment this out if it is too intensive (not required for the xml spec)
      $select = db_select('mm_tree', 't');
      $select->leftJoin('mm_tree_revisions', 'r', 'r.vid = t.vid');
      $select->leftJoin('mm_node2tree', 'm', 't.mmtid = m.mmtid');
      $select->leftJoin('node', 'n', 'm.nid = n.nid');
      $select->condition('t.mmtid', $item->mmtid)
        ->groupBy('t.mmtid')
        ->groupBy('r.mtime');
      $select->addField('r', 'mtime', 't_changed');
      $select->addExpression('MAX(n.changed)', 'n_changed');
      $mod_time = $select->execute()->fetchAssoc();
      // End of last updated section
      $max_mod_time = max($mod_time['t_changed'], $mod_time['n_changed']);

      fwrite($this->fp, "<url>\n");
      fwrite($this->fp, "\t<loc>" . htmlspecialchars($base_url . $path, ENT_QUOTES) . "</loc>\n");
      if (!is_null($max_mod_time)) {
        fwrite($this->fp, "\t<lastmod>" . date('Y-m-d', $max_mod_time) . "</lastmod>\n");
      }
      // fwrite($this->fp, "\t<priority>".round(1/$item->level, 2)."</priority>\n");
      fwrite($this->fp, "</url>\n");

      return 1;       // no error
    }

    public function finish() {
      fwrite($this->fp, "</urlset>\n");
      fclose($this->fp);
      $source = $this->file_path . '.temp';   // necessary for pass-by-reference
      file_move($source, $this->file_path, FILE_EXISTS_REPLACE);
    }
  }

  $iter = new dumpIter();
  // Use the anonymous user, so permissions tests are valid
  $params = array(
    MM_GET_TREE_FILTER_NORMAL => TRUE,
    MM_GET_TREE_ITERATOR      => $iter,
    MM_GET_TREE_RETURN_BLOCK  => TRUE,
    MM_GET_TREE_RETURN_PERMS  => TRUE,
    MM_GET_TREE_USER          => user_load(0),
  );
  mm_content_get_tree(1, $params);
  $iter->finish();
  exit();
}

function mm_sitemap_show() {
  file_transfer('sitemap.xml', array('Content-Type: text/xml'));
}
